package lrs.visitor;
import lrs.*;

/**
 * Represents an abstract algorithm on an LRStruct.  Acts as a visitor to a
 * LRStruct host.
 * @author Dung X. Nguyen and Stephen Wong  Copyright 2005 - All rights reserved.
 * @since 8/25/05
 */
public class Sum implements IAlgo<Number, Double, Object> {
  
  public static final Sum Singleton = new Sum();
  private Sum(){}
    /**
    * Operates on an empty LRStruct host, given an input object.
    * @param host an empty LRStruct.
    * @param inp variable input list of objects needed by this IVisitor.
    * @return an appropriate output object.
    */
  public Double emptyCase(LRStruct<? extends Number> host, Object... inp) {
    return 0.0;
  }

    /**
    * Operates on a non-empty LRStruct host, given an input object.
    * @param host a non-empty LRStruct.
    * @param inp variable input list of objects needed by this IVisitor.
    * @return an appropriate output object.
    */
  public Double nonEmptyCase(LRStruct<? extends Number> host, Object... inp) {
    return host.getFirst().doubleValue() + host.getRest().execute(this);
  }
}

